#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
#include <string.h>

// definicje pinow przekaznikow
#define R1 13
#define R2 14
#define R3 5
#define R4 4

// ustawienia parametrow sieci
const char *ssid = "b97488";
const char *password = "280740688";

ESP8266WebServer server ( 80 );

uint8_t tablica_relays[] = { '0', '0', '0', '0'};

void przekazniki_zapal( void )
{
  if ( tablica_relays[ 0 ] == '0' )
  {
    digitalWrite( R1, LOW );
  }
  else
  {
    digitalWrite( R1, HIGH );
  }
  
  if ( tablica_relays[ 1 ] == '0' )
  {
    digitalWrite( R2, LOW );
  }
  else
  {
    digitalWrite( R2, HIGH );
  }
  
  if ( tablica_relays[ 2 ] == '0' )
  {
    digitalWrite( R3, LOW );
  }
  else
  {
    digitalWrite( R3, HIGH );
  }
  
  if ( tablica_relays[ 3 ] == '0' )
  {
    digitalWrite( R4, LOW );
  }
  else
  {
    digitalWrite( R4, HIGH );
  }
}

const String strona =
"<html>\
  <head>\
    <title>ESP RELAY</title>\
    <style>\
      body { background-color: #cccccc; font-family: Arial, Helvetica, Sans-Serif; Color: #000088; }\
    </style>\
  </head>\
  <body>\
<p>Przekaznik nr 1:</p>\
<form action=\"relays\?rel1=on\" method=\"POST\">\
    <input type=\"submit\" value=\"Zapal\">\
</form>\
<form action=\"relays\?rel1=off\" method=\"POST\">\
    <input type=\"submit\" value=\"Zgas\">\
</form>\
<p>Przekaznik nr 2:</p>\
<form action=\"relays\?rel2=on\" method=\"POST\">\
    <input type=\"submit\" value=\"Zapal\">\
</form>\
<form action=\"relays\?rel2=off\" method=\"POST\">\
    <input type=\"submit\" value=\"Zgas\">\
</form>\
<p>Przekaznik nr 3:</p>\
<form action=\"relays\?rel3=on\" method=\"POST\">\
    <input type=\"submit\" value=\"Zapal\">\
</form>\
<form action=\"relays\?rel3=off\" method=\"POST\">\
    <input type=\"submit\" value=\"Zgas\">\
</form>\
<p>Przekaznik nr 4:</p>\
<form action=\"relays\?rel4=on\" method=\"POST\">\
    <input type=\"submit\" value=\"Zapal\">\
</form>\
<form action=\"relays\?rel4=off\" method=\"POST\">\
    <input type=\"submit\" value=\"Zgas\">\
</form>\
  </body>\
</html>";
  
void handleRoot() 
{
	server.send ( 200, "text/html", strona );
}

void handleNotFound() {
	String message = "Site Not Found\n\n";

	server.send ( 404, "text/plain", message );
}

void przekazniki_funkcja( void )
{
    String message = "Otrzymalem:\n";
    message += "URI: ";
    message += server.uri();
    message += "\nMethod: ";
    message += ( server.method() == HTTP_GET ) ? "GET" : "POST";
    message += "\nArguments: ";
    message += server.args();
    message += "\n";

    // debug na konsoli
    for ( uint8_t i = 0; i < server.args(); i++ ) {
      message += " " + server.argName ( i ) + ": " + server.arg ( i ) + "\n";
    }
    Serial.println ( message );

    // rozkodowanie komendy
    for ( uint8_t i = 0; i < server.args(); i++ ) 
    {
      if( server.argName( i )[ 0 ] == 'r' && server.argName( i )[ 1 ] == 'e' && server.argName( i )[ 2 ] == 'l' )
      {
        // zabezpieczenie przed przekroczeniem indexu
        if( server.argName( i )[ 3 ] - '1'  < 4 )
        {
          tablica_relays[ server.argName( i )[ 3 ] - '1' ] = (server.arg( i ) == "on" ? '1' : '0');
        }
      }
    }
    // wykonanie akcji
    przekazniki_zapal();

    // przeslanie strony glownej
    handleRoot();
}

void json_funkcja( void )
{
    String odpowiedz = "{\"rel1\":\"";
    odpowiedz += ( tablica_relays[0] == '0' ) ? "off" : "on";
    odpowiedz += "\",\"rel2\":\"";
    odpowiedz += ( tablica_relays[1] == '0' ) ? "off" : "on";
    odpowiedz += "\",\"rel3\":\"";
    odpowiedz += ( tablica_relays[2] == '0' ) ? "off" : "on";
    odpowiedz += "\",\"rel4\":\"";
    odpowiedz += ( tablica_relays[3] == '0' ) ? "off" : "on";
    odpowiedz += "\"}";
    server.send ( 200, "text/html", odpowiedz );
}

void setup ( void ) 
{
  // ustawienie pinow w stan niski
  digitalWrite( R1, LOW );
  digitalWrite( R2, LOW );
  digitalWrite( R3, LOW );
  digitalWrite( R4, LOW );

  // inicjalizacja pinow jako wyjscia
	pinMode ( R1, OUTPUT );
  pinMode ( R2, OUTPUT );
  pinMode ( R3, OUTPUT );
  pinMode ( R4, OUTPUT );

  // stan domyslny przekaznikow z tablicy
  przekazniki_zapal();

  // ustawienie uartu dla debugowania
	Serial.begin ( 115200 );

  // polaczenie z siecia
	WiFi.begin ( ssid, password );
	Serial.println ( "" );

	// Oczekiwanie na polaczenie
	while ( WiFi.status() != WL_CONNECTED ) {
		delay ( 500 );
		Serial.print ( "." );
	}

  // wyswietlenie informacji o polaczeniu
	Serial.println ( "" );
	Serial.print ( "Connected to " );
	Serial.println ( ssid );
	Serial.print ( "IP address: " );
	Serial.println ( WiFi.localIP() );
  
	if ( MDNS.begin ( "esp8266" ) ) 
	{
		Serial.println ( "MDNS responder started" );
	}

  // przypisanie funkcji do danych jakie moze odebrac serwer
	server.on ( "/", handleRoot );
  server.on( "/relays", przekazniki_funkcja );
  server.on( "/rel", json_funkcja );
	server.onNotFound( handleNotFound );

  // uruchomienie serwera
	server.begin();

 // wyswietlenie informacji o rurchomieniu
	Serial.println ( "HTTP server started" );
}

void loop ( void ) 
{
	// obsluga klientow przez serwer
	server.handleClient();
}

